# actions.py

import scummpy


CANCELLED = const(-1)
FINISHED = const(0)
RUNNING = const(1)

class Wait_Action(object):
  def __init__(self, ticks, blocking=True):
    self.blocking = blocking
    
    self._ticks = ticks
  
  
  def init(self):
    pass
  
  
  def abort(self):
    pass
  
  
  def update(self):
    if self._ticks <= 0:
      return False
    self._ticks -= 1
    return True
  
  
  def draw(self):
    pass


class Look_Action(object):
  def __init__(self, actor, x, blocking=False):
    self.blocking = blocking
    
    self._actor = actor
    self._target_x = x
  
  
  def init(self):
    if self._target_x < self._actor.entity.x:
      self._actor.direction = 8
    elif self._target_x > self._actor.entity.x:
      self._actor.direction = 2
  
  
  def abort(self):
    pass
  
  
  def update(self):
    return False
  
  
  def draw(self):
    pass


class Walk_Action(object):
  def __init__(self, actor, target_x, target_y, blocking=False):
    self.blocking = blocking
    
    self._actor = actor
    self._target_x = target_x
    self._target_y = target_y
  
  
  def init(self):
    if self._target_x != self._actor.entity.x:
      if self._target_x < self._actor.entity.x:
        self._actor.direction = 8
      elif self._target_x > self._actor.entity.x:
        self._actor.direction = 2
      self._actor.start_walk_anim()
    elif self._target_y != self._actor.entity.y:
      self._actor.start_walk_anim()
  
  
  def abort(self):
    self._actor.start_idle_anim()
  
  
  def update(self):
    cur_x = self._actor.entity.x
    cur_y = self._actor.entity.y
    dx = self._target_x-cur_x
    dy = self._target_y-cur_y
    sign_dx = int(dx > 0)-int(dx < 0)
    if dy == 0 and sign_dx*dx >= 2:
      dx = 2*sign_dx
    else:
      dx = sign_dx
    dy = int(dy > 0)-int(dy < 0)
    
    dx -= self._check_collision(cur_x, cur_y, dx, 0)
    dy -= self._check_collision(cur_x+dx, cur_y, 0, dy)
    self._actor.entity.x += dx
    self._actor.entity.y += dy

    if cur_x+dx == self._target_x and cur_y+dy == self._target_y:
      self._actor.start_idle_anim()
      return False
    elif dx == 0 and dy == 0:
      self._actor.start_idle_anim()
      return False
    
    return True
  
  
  def draw(self):
    pass
  
  
  def _check_collision(self, x, y, dx, dy):
    map_x = (x+dx)>>3
    map_y = (y+dy)>>3
    if not scummpy.room.is_tile_walkable(map_x, map_y):
      if dx != 0:
        modulo = x+dx-(map_x<<3)
        return modulo+1 if dx > 0 else modulo-8
      elif dy != 0:
        modulo = y+dy-(map_y<<3)
        return modulo+1 if dy > 0 else modulo-8
    
    return 0


class Examine_Action(object):
  def __init__(self, actor, entity, blocking=True):
    self.blocking = blocking
    
    self._actor = actor
    self._entity = entity
  
  
  def init(self):
    pass
  
  
  def abort(self):
    pass
  
  
  def update(self):
    if self._entity.examine:
      self._entity.examine(self._actor)
    return False
  
  
  def draw(self):
    pass


class Interact_Action(object):
  def __init__(self, actor, entity, blocking=True):
    self.blocking = blocking
    
    self._actor = actor
    self._entity = entity
  
  
  def init(self):
    pass
  
  
  def abort(self):
    pass
  
  
  def update(self):
    if self._entity.interact:
      self._entity.interact(self._actor)
    return False
  
  
  def draw(self):
    pass


class UseWith_Action(object):
  def __init__(self, entity1, entity2, blocking=True):
    self.blocking = blocking
    
    self._entity1 = entity1
    self._entity2 = entity2
  
  
  def init(self):
    pass
  
  
  def abort(self):
    pass
  
  
  def update(self):
    if self._entity2.use_with:
      self._entity2.use_with(self._entity1)
    return False
  
  
  def draw(self):
    pass


class Talk_Action(object):
  def __init__(self, actor, lines, ticks, blocking=False):
    self.blocking = blocking
    
    self._actor = actor
    self._lines = lines if type(lines) == list else [lines]
    self._ticks = ticks
  
  
  def init(self):
    if self._actor:
      self._actor.start_talk_anim()
  
  
  def abort(self):
    if self._actor:
      self._actor.start_idle_anim()
  
  
  def update(self):
    if self._ticks <= 0:
      if self._actor:
        self._actor.start_idle_anim()
      return False
    self._ticks -= 1
    return True
  
  
  def draw(self):
    y0 = self._actor.entity.y+self._actor.entity.draw_y if self._actor else 40
    y0 -= len(self._lines)*7
    if y0 < 1:
      y0 = 1
    for line_num in range(len(self._lines)):
      text = self._lines[line_num]
      x = self._actor.entity.x if self._actor else 55
      x -= (len(text)*4>>1)
      color = self._actor.color if self._actor else 11
      scummpy.draw_outlined_text(text, x, y0+line_num*7, color)


class PickUp_Action(object):
  def __init__(self, actor, entity, blocking=True):
    self.blocking = blocking
    
    self._actor = actor
    self._entity = entity
  
  
  def init(self):
    for idx in range(len(scummpy.room.entities)):
      if scummpy.room.entities[idx] == self._entity:
        scummpy.room.entities.pop(idx)
        self._actor.direction = 8 if self._entity.x < self._actor.entity.x else 2
        break
    self._actor.add_item(self._entity)
  
  
  def abort(self):
    pass
  
  
  def update(self):
    return False
  
  
  def draw(self):
    pass


class Drop_Action(object):
  def __init__(self, actor, entity, blocking=True):
    self.blocking = blocking
    
    self._actor = actor
    self._entity = entity
  
  
  def init(self):
    self._actor.remove_item(self._entity)
  
  
  def abort(self):
    pass
  
  
  def update(self):
    return False
  
  
  def draw(self):
    pass


class SwitchPlayer_Action(object):
  def __init__(self, target_actor, blocking=True):
    self.blocking = blocking
    
    self._target_actor = target_actor
  
  
  def init(self):
    scummpy.player = self._target_actor
  
  
  def abort(self):
    pass
  
  
  def update(self):
    return False
  
  
  def draw(self):
    pass


