# ui.py

import upygame
import umachine
import ui_data
from actions import Look_Action
from actions import Walk_Action
from actions import Examine_Action
from actions import Interact_Action
from actions import UseWith_Action


screen = None
room = None
player = None
inventory_shown = True

active_action = None
pending_actions = []

btn_states = 0    # bitarray of button states
btn_pressed = 0   # button code if a button press was detected in last update

BTN_UP = const(1)
BTN_RIGHT = const(2)
BTN_DOWN = const(4)
BTN_LEFT = const(8)
BTN_A = const(16)
BTN_B = const(32)
BTN_C = const(64)

mouse_x = 55
mouse_y = 44
mouse_room = None
mouse_target = None

AREA_ROOM = const(1)
AREA_INVENTORY = const(2)

_mouse_dx = 0
_mouse_dy = 0

_cursor_anim = [
  ui_data.cursor_0, ui_data.cursor_1, ui_data.cursor_2,
  ui_data.cursor_2, ui_data.cursor_1, ui_data.cursor_0
]
_CURSOR_TRANSP_COLOR = const(0)

_TEXT_OUTLINE_COLOR = const(1)


def update():
  global room, player, active_action, pending_actions, mouse_x, mouse_y
  
  _buttons_update()
  _mouse_update()
  _find_mouse_target()
  
  if btn_pressed&0x30:
    if mouse_area == AREA_ROOM:
      if flush_action():
        if mouse_target:
          ent = mouse_target
          if btn_pressed&0x10:
            pending_actions.append(Walk_Action(player, ent.x+ent.act_x, ent.y+ent.act_y, True))
            pending_actions.append(Look_Action(player, ent.x))
            if player.active_item:
              pending_actions.append(UseWith_Action(player.active_item, ent))
            else:
              pending_actions.append(Interact_Action(player, ent))
          elif btn_pressed&0x20 and not player.active_item:
            pending_actions.append(Walk_Action(player, ent.x+ent.act_x, ent.y+ent.act_y, True))
            pending_actions.append(Look_Action(player, ent.x))
            pending_actions.append(Examine_Action(player, ent))
        else:
          if not player.active_item:
            pending_actions.append(Walk_Action(player, mouse_x, mouse_y, False))
        player.active_item = None
    elif mouse_area == AREA_INVENTORY:
      if btn_pressed&0x10:
        if flush_action():
          if player.active_item and mouse_target and player.active_item != mouse_target:
            pending_actions.append(UseWith_Action(player.active_item, mouse_target))
            player.active_item = None
          else:
            player.active_item = mouse_target
      elif btn_pressed&0x20:
        if flush_action():
          if not player.active_item and mouse_target:
            pending_actions.append(Examine_Action(player, mouse_target))
        player.active_item = None
  
  if active_action:
    if not active_action.update():
      active_action = None
  
  if not active_action and len(pending_actions) > 0:
    active_action = pending_actions.pop(0)
    active_action.init()
  
  for actor in room.actors:
    actor.update()
  


def _entity_z_order(entity):
  return entity.z_order


def render():
  global screen, room, mouse_x, mouse_y, _cursor_anim
  
  room.tilemap.draw(0, 0)
  
  entities = room.entities + [actor.entity for actor in room.actors]
  entities.sort(key=_entity_z_order)
  for ent in entities:
    ent.draw(screen)
  
  if active_action:
    active_action.draw()
  
  _draw_inventory()
  
  # Draw mouse cursor
  frame = (umachine.time_ms()//100)%len(_cursor_anim)
  sprite = _cursor_anim[frame]
  screen.blit(sprite, mouse_x-3, mouse_y-3, _CURSOR_TRANSP_COLOR)
  
  upygame.display.flip()


def flush_action():
  global active_action, pending_actions
  
  ready_for_action = True
  if active_action and not active_action.blocking:
    ready_for_action = isinstance(active_action, Walk_Action)
    active_action.abort()
    active_action = None
  # Return true if new action will be accepted right away with the same button click
  return ready_for_action and not active_action and len(pending_actions) == 0


def change_room(target_room, x, y, direction):
  global room, player
  
  if room:
    if room.exit:
      room.exit()
    
    for idx in range(len(room.actors)):
      if room.actors[idx] == player:
        room.actors.pop(idx)
        break
  
  room = target_room
  room.actors.append(player)
  
  player.entity.x = x
  player.entity.y = y
  player.direction = direction
  
  if room.enter:
    room.enter()


def _draw_inventory():
  global screen, player
  
  if not inventory_shown:
    return
  
  screen.fill(0, upygame.Rect(0, 72, 110, 15))
  
  text = "Usa "+player.active_item.name+" con " if player.active_item else ""
  if mouse_target and mouse_target != player.active_item:
    text += mouse_target.name
  x = 55-4*(len(text)>>1)
  if x < 0:
    x = 0
  screen.blit(None, 0, 0, 0)   # Set color 0 transparent
  umachine.draw_text(x, 73, text, 7)
  
  screen.blit(ui_data.inventory_arrow_l, 0, 78, 255)
  for idx in range(10):
    screen.blit(ui_data.inventory_slot, 6+10*idx, 79, 255)
  screen.blit(ui_data.inventory_arrow_r, 105, 78, 255)
  
  for idx in range(len(player.inventory)):
    ent = player.inventory[idx]
    screen.blit(ent.sprite, 6+10*idx, 79, ent.transp_color)


def draw_outlined_text(text, x, y, color):
  global screen
  
  offset_arr = b'\x10\x20\x21\x22\x12\x02\x01\x00'
  if x < 1:
    x = 1
  elif x >= 110-(len(text)*4+1):
    x = 110-(len(text)*4+1)
  if y < 1:
    y = 1
  x -= 1
  y -= 1
  
  screen.blit(None, 0, 0, 0)   # Set color 0 transparent
  for offset in offset_arr:
    umachine.draw_text(x+(offset>>4), y+(offset&0xf), text, _TEXT_OUTLINE_COLOR)
  umachine.draw_text(x+1, y+1, text, color)


def _buttons_update():
  global btn_states, btn_pressed
  
  btn_pressed = 0
  eventtype = upygame.event.poll()
  if eventtype != upygame.NOEVENT:
    if eventtype.type == upygame.KEYDOWN:
      if eventtype.key == upygame.K_UP:    btn_pressed = 0x01
      if eventtype.key == upygame.K_RIGHT: btn_pressed = 0x02
      if eventtype.key == upygame.K_DOWN:  btn_pressed = 0x04
      if eventtype.key == upygame.K_LEFT:  btn_pressed = 0x08
      if eventtype.key == upygame.BUT_A:   btn_pressed = 0x10
      if eventtype.key == upygame.BUT_B:   btn_pressed = 0x20
      if eventtype.key == upygame.BUT_C:   btn_pressed = 0x40
      btn_states |= btn_pressed
    if eventtype.type == upygame.KEYUP:
      if eventtype.key == upygame.K_UP:    btn_states &= ~0x01
      if eventtype.key == upygame.K_RIGHT: btn_states &= ~0x02
      if eventtype.key == upygame.K_DOWN:  btn_states &= ~0x04
      if eventtype.key == upygame.K_LEFT:  btn_states &= ~0x08
      if eventtype.key == upygame.BUT_A:   btn_states &= ~0x10
      if eventtype.key == upygame.BUT_B:   btn_states &= ~0x20
      if eventtype.key == upygame.BUT_C:   btn_states &= ~0x40


def _mouse_update():
  global btn_states, mouse_x, mouse_y, _mouse_dx, _mouse_dy

  val = btn_states&10
  if val == 2:
    _mouse_dx = _mouse_dx+1 if _mouse_dx < 10 else 10
  elif val == 8:
    _mouse_dx = _mouse_dx-1 if _mouse_dx > -10 else -10
  else:
    _mouse_dx = 0
  
  val = btn_states&5
  if val == 1:
    _mouse_dy = _mouse_dy-1 if _mouse_dy > -10 else -10
  elif val == 4:
    _mouse_dy = _mouse_dy+1 if _mouse_dy < 10 else 10
  else:
    _mouse_dy = 0
  
  mouse_x += _mouse_dx
  if mouse_x < 0:
    mouse_x = 0
  elif mouse_x >= 110:
    mouse_x = 109
  
  mouse_y += _mouse_dy
  if mouse_y < 0:
    mouse_y = 0
  elif mouse_y >= 88:
    mouse_y = 87


def _find_mouse_target():
  global room, player, inventory_shown, mouse_area, mouse_target, mouse_x, mouse_y, ROOM_AREA, INVENTORY_AREA
  
  mouse_room = None
  mouse_target = None
  
  if mouse_y < 72 or not inventory_shown:
    mouse_area = AREA_ROOM
    entities = room.entities + [actor.entity for actor in room.actors if actor != player]
    for ent in entities:
      if ent.clickable:
        left = ent.x+ent.draw_x
        top = ent.y+ent.draw_y
        right = left+ent.sprite.get_rect().width
        bottom = top+ent.sprite.get_rect().height
        if mouse_x >= left and mouse_x < right:
          if mouse_y >= top and mouse_y < bottom:
            mouse_target = ent
            return
  else:
    mouse_area = AREA_INVENTORY
    if mouse_y >= 79 and mouse_y < 87:
      for idx in range(len(player.inventory)):
        left = 6+10*idx
        right = left+8
        if mouse_x >= left and mouse_x < right:
          mouse_target = player.inventory[idx]
          return

